from scapy.utils import rdpcap
from scapy.layers.inet import IP, TCP, UDP
import pandas as pd

# Load PCAP file
pcap_path = '/mnt/300gb/iot-5G-project/data/2018-12-21-15-50-14-192.168.1.195.pcap'
packets = rdpcap(pcap_path)

# Initialize a list to store extracted features
flow_data = []

# Process each packet
for packet in packets:
    if IP in packet:
        flow = {
            "src_ip": packet[IP].src,
            "dst_ip": packet[IP].dst,
            "protocol": packet[IP].proto,
            "packet_length": len(packet),
            "timestamp": packet.time
        }
        if TCP in packet or UDP in packet:
            flow["src_port"] = packet.sport
            flow["dst_port"] = packet.dport
        flow_data.append(flow)

# Convert to DataFrame for feature extraction
flows_df = pd.DataFrame(flow_data)

# Group by flows (src_ip, dst_ip, protocol) to aggregate features
aggregated_flows = flows_df.groupby(["src_ip", "dst_ip", "protocol"]).agg({
    "packet_length": ["sum", "count"],
    "timestamp": ["min", "max"]
}).reset_index()

# Flatten multi-level columns
aggregated_flows.columns = ["_".join(col).strip("_") for col in aggregated_flows.columns]

# Add flow duration
aggregated_flows["flow_duration"] = aggregated_flows["timestamp_max"] - aggregated_flows["timestamp_min"]

# Save as CSV for further analysis
aggregated_flows.to_csv('results/extracted_flows.csv', index=False)
print("Feature extraction complete. Saved to results/extracted_flows.csv")
